/**************************************************************************************************
*
* * Project name:
*     LDC1000 click
* * Copyright:
*     (c) mikroElektronika, 2014.
* * Revision History:
*     20140812:
*     - Initial release (DB);
* Description:
*     LDC1000 click carries the world's first inductance-to-digital converter IC,
*     along with a detachable sensor (an LC tank comprising a 36-turn PCB coil and a 100pF 1% NPO capacitor).
*     The LDC1000 IC has a sub-micron resolution in short range applications;
*     the board is ideal for highly precise short range measurements of the position,
*     motion or composition of conductive targets. Inductive sensing is highly reliable:
*     harsh conditions don't hinder the performance of LDC1000 IC.
*     Onboard INA and INB pins allow you to replace the provided detachable sensor and solder your own.
*     LDC1000 click communicates with the target board through mikroBUS SPI (CS, SCK, MISO, MOSI) and INT lines.
*     The board is designed to use either a 3.3V or a 5V power supply.
*     Example send data on USB UART A port. Use USART Terminal from Tools menu.
*     With our experiments we achieved operational range between 2mm and 1,7cm with stainless steel material.
* Test configuration:
*     MCU:             STM32F107VC
*                      http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/DATASHEET/CD00220364.pdf
*     Dev.Board:       EasyMx PRO v7 for STM32(R) ARM(R)
*                      http://www.mikroe.com/easymx-pro/stm32/
*     Oscillator:      HSE-PLL, 72.000MHz
*     SW:              mikroC PRO for ARM
*                      http://www.mikroe.com/mikroc/arm/
* NOTES:
*     - Place LDC1000 click board in the mikroBUS socket 1.
*     - Enable SW12-1 PA9 and SW12-2 PA10 (USB UART A)
**************************************************************************************************/



sbit Chip_Select at GPIOD_ODR.B13;

void Init_System(){
    GPIO_Digital_Output(&GPIOD_ODR, _GPIO_PINMASK_13);      // Set GPIOD pin 13 output
    
    SPI3_Init_Advanced(_SPI_FPCLK_DIV8, _SPI_MASTER | _SPI_8_BIT |         //// Initialize SPI3 module
                     _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION |
                     _SPI_MSB_FIRST | _SPI_SS_DISABLE | _SPI_SSM_ENABLE | _SPI_SSI_1,
                     &_GPIO_MODULE_SPI3_PC10_11_12);
    //Delay_ms(100);
    UART1_Init(56000);          // Initialize UART module at 56000 bps
    Delay_ms(100);              // Wait for UART module to stabilize
}
// Write data to an registry address
void LDC1000_Reg_Write(char reg_address, char reg_value){
     Chip_Select=0;
     SPI3_Write(reg_address);
     SPI3_Write(reg_value);
     Chip_Select=1;
}
//Read registry data and send it to uart
void LDC1000_Reg_Read(char reg_address, char Text[10]){
    short temp;
    char txt[3];
    Chip_Select=0;
    SPI3_Read(reg_address);
    temp=SPI3_Read(0xFF);
    Chip_Select=1;
    ShortToHex(temp, txt);
    UART1_Write_Text(Text);
    UART1_Write_Text(txt);
    UART1_Write(13);
    UART1_Write(10);
}

void Init_LDC1000() {

//Rp_MAX
        LDC1000_Reg_Write(0x01,0x11);  // Rp_Max=38.785K
//Rp_MIN
        LDC1000_Reg_Write(0x02,0x3B);  // Rp_Min=2.39K
//Sensor Frequency
        LDC1000_Reg_Write(0x03,0xD9);  // ==> N = 68.94 * log10(F/2000)=217 <==;  F = 20% below resonating frequency, Hz; N = Register Value (Round to nearest value)
//LDC Configuration
        LDC1000_Reg_Write(0x04,0x0F);  // Amplitude=2V & Response Time=6144;
//Clock Configuration
        LDC1000_Reg_Write(0x05,0x02);  //External Crystal used for Frequency Counter; Enable External time base clock;
/*
//Comparator Threshold High LSB
        LDC1000_Reg_Write(0x06,0x7C);
//Comparator Threshold High MSB
        LDC1000_Reg_Write(0x07,0x1A);
//Comparator Threshold Low LSB
        LDC1000_Reg_Write(0x08,0x1D);
//Comparator Threshold Low MSB
        LDC1000_Reg_Write(0x09,0x10);
//INTB Pin Configuration
        LDC1000_Reg_Write(0x0A,0x02);
*/
//Power Configuration
        LDC1000_Reg_Write(0x0B,0x01);  //Active Mode, Conversion is Enabled
}

void LDC1000_Reg_Check(){

  UART1_Write_Text("Register map:");
  UART1_Write(13);
  UART1_Write(10);

//Rp_MAX
        LDC1000_Reg_Read(0x81, "Rp_MAX = 0x");
//Rp_MIN
        LDC1000_Reg_Read(0x82, "Rp_MIN = 0x");
//Sensor Frequency
         LDC1000_Reg_Read(0x83, "Sensor freq = 0x");
//LDC Configuration
      LDC1000_Reg_Read(0x84, "LDC conf = 0x");
//Clock Configuration
        LDC1000_Reg_Read(0x85, "Clock conf = 0x");
/*
//Threshold High LSB
            LDC1000_Reg_Read(0x86, "Threshold High LSB = 0x");
//Threshold High MSB
            LDC1000_Reg_Read(0x87, "Threshold High MSB = 0x");
//Threshold low LSB
            LDC1000_Reg_Read(0x88, "Threshold low LSB = 0x");
//Threshold low MSB
            LDC1000_Reg_Read(0x89, "Threshold low MSB = 0x");
*/
//INTB Pin Configuration
       LDC1000_Reg_Read(0x8A,"INTB_Pin = 0x");
//Power Configuration
        LDC1000_Reg_Read(0x8B, "Pow conf = 0x");
//Status
        LDC1000_Reg_Read(0xA0, "Status = 0x");
}
//Measuring Inductance
void Induct_Measure() {
     unsigned long Fcount, prox, temp, temp1;
     char txt1[21], txt2[15];
     float ind, fsens;

//reading Proximity Data register
    Chip_Select=0;
    SPI3_Read(0xA1);
    prox=SPI3_Read(0xFF);
    Chip_Select=1;

    Chip_Select=0;
    SPI3_Read(0xA2);
    temp1=SPI3_Read(0xFF);
    Chip_Select=1;
    temp1=temp1<<8;
    prox=prox+temp1;

    LongLongUnsignedToStr(prox, txt1);
    UART1_Write_Text("Proximity Data = ");
    UART1_Write_Text(txt1);
    UART1_Write(13);
    UART1_Write(10);
    Delay_ms(100);

//reading Frequency Counter register and calculating Frequency
     Chip_Select=0;
     SPI3_Read(0xA3);
     Fcount=SPI3_Read(0xFF);
     Chip_Select=1;

     Chip_Select=0;
     SPI3_Read(0xA4);
     temp=SPI3_Read(0xFF);
     Chip_Select=1;
     temp=temp<<8;
     Fcount=Fcount+temp;

     Chip_Select=0;
     SPI3_Read(0xA5);
     temp=SPI3_Read(0xFF);
     Chip_Select=1;
     temp=temp<<16;
     Fcount=Fcount+temp;
     //Measuring Sensor frequency
     //fsens = (1/3)*(Fext/Fcount)*(Response Time)
     //Fext is the frequency of the external clock (8MHz)
     //Fcount is the value obtained from the Frequency Counter Data register(address 0x23,0x24,0x25)
     //Response Time is the programmed response time (see LDC configuration register, address 0x04)
     fsens=(1./3.)*(8000000./Fcount)*(6144);
      
      //Measuring Sensor inductance
     ind=1e6/(100e-12*((2.*3.14*fsens)*(2.*3.14*fsens)));

     FloatToStr(ind, txt2);
     UART1_Write_Text("Impedance L = ");
     UART1_Write_Text(txt2);
     UART1_Write_Text(" uH");
     UART1_Write(13);
     UART1_Write(10);
     UART1_Write(13);
     UART1_Write(10);
     Delay_ms(100);
}

void main() {
  Init_System();
  Init_LDC1000();         // Initialize LDC1000 Registers
  LDC1000_Reg_Check();         // Read LDC1000 Registers
           do{
             Induct_Measure();
             Delay_ms(500);
             }while(1);
  }